@@ -5,21 +5,21 @@ from abc import ABC, abstractmethod
 from dataclasses import asdict
 import io
 from operator import itemgetter
-import warnings
 from collections import defaultdict
 import json
+from typing import Dict, Union
 
 import six
 from blazeutils.functional import identity
 from markupsafe import Markup
 
 from blazeutils.containers import HTMLAttributes, LazyDict
-from blazeutils.datastructures import BlankObject
 from blazeutils.helpers import tolist
 from blazeutils.jsonh import jsonmod
-from blazeutils.spreadsheets import Writer, WriterX, xlsxwriter
+from blazeutils.spreadsheets import WriterX, xlsxwriter, openpyxl
 from blazeutils.strings import reindent, randnumerics
 import jinja2 as jinja
+
 from werkzeug.datastructures import MultiDict
 from werkzeug.routing import Map, Rule
 
@@ -33,6 +33,10 @@ from .utils import current_url
 from . import extensions, types
 import csv
 
+if openpyxl:
+    from openpyxl.styles import Font, Border, Side, Alignment
+    from openpyxl.utils import get_column_letter
+
 try:
     from morphi.helpers.jinja import configure_jinja_environment
 except ImportError:
@@ -43,11 +47,6 @@ try:
 except ImportError:
     is_lazy_string = lambda value: False  # noqa: E731
 
-try:
-    import xlwt
-except ImportError:
-    xlwt = None
-
 
 def fix_xls_value(value):
     """
@@ -1171,10 +1170,6 @@ class HTML(GroupMixin, Renderer):
             renderer (str): Export key (e.g. xlsx, csv) for rendering target."""
         return self.current_url(export_to=renderer)
 
-    def xls_url(self):
-        warnings.warn('xls_url is deprecated. Use export_url instead.', DeprecationWarning)
-        return self.export_url('xls')
-
     def _get_filter_select_info(self):
         return _('Add Filter:'), self.filtering_add_filter_select()
 
@@ -1219,183 +1214,26 @@ class HTML(GroupMixin, Renderer):
         )
 
 
-class XLS(Renderer):
-    mime_type = 'application/vnd.ms-excel'
-
-    @property
-    def name(self):
-        return 'xls'
-
-    def __init__(self, grid, max_col_width=150):
-        warnings.warn(
-            "XLS support is deprecated and will be removed in a future version",
-            DeprecationWarning
-        )
-        super().__init__(grid)
-        self.define_styles()
-        self.col_contents_widths = defaultdict(int)
-        self.max_col_width = max_col_width
-
-    def render(self):
-        return self.build_sheet()
-
-    def can_render(self):
-        total_rows = self.grid.record_count + 1
-        if self.grid.subtotals != 'none':
-            total_rows += 1
-        return total_rows <= 65536 and sum(1 for _ in self.columns) <= 256
-
-    def define_styles(self):
-        self.style = BlankObject()
-        self.style.bold = xlwt.easyxf('font: bold True;')
-
-    def sanitize_sheet_name(self, sheet_name):
-        return sheet_name if len(sheet_name) <= 30 else (sheet_name[:27] + '...')
-
-    def build_sheet(self, wb=None, sheet_name=None):
-        if xlwt is None:
-            # !!!: translate?
-            raise ImportError('you must have xlwt installed to use Excel renderer')
-
-        if not self.can_render():
-            raise RenderLimitExceeded('Unable to render XLS sheet')
-
-        if wb is None:
-            wb = xlwt.Workbook()
-        sheet = wb.add_sheet(
-            self.sanitize_sheet_name(sheet_name or self.grid.ident)
-        )
-        xlh = Writer(sheet)
-
-        self.sheet_header(xlh)
-        self.sheet_body(xlh)
-        self.sheet_footer(xlh)
-        self.adjust_col_widths(sheet)
-
-        return wb
-
-    def sheet_header(self, xlh):
-        pass
-
-    def sheet_body(self, xlh):
-        self.body_headings(xlh)
-        self.body_records(xlh)
-
-    def sheet_footer(self, xlh):
-        pass
-
-    def register_col_width(self, col, value):
-        if value is None:
-            return
-        self.col_contents_widths[col.key] = max(
-            self.col_contents_widths[col.key],
-            col.xls_width_calc(value)
-        )
-
-    def adjust_col_widths(self, ws):
-        for idx, col in enumerate(self.columns):
-            max_registered_width = self.col_contents_widths[col.key]
-            final_width = min(max_registered_width, self.max_col_width)
-
-            # width calculation is 1/256th of width of zero character, using the
-            # first font that occurs in the Excel file
-            # the following calculation seems to get it done alright
-            ws.col(idx).width = int((final_width + 3) * 256)
-
-    def body_headings(self, xlh):
-        for col in self.columns:
-            self.register_col_width(col, col.label)
-            xlh.awrite(fix_xls_value(col.label), self.style.bold)
-        xlh.newrow()
-
-    def body_records(self, xlh):
-        # turn off paging
-        self.grid.set_paging(None, None)
-
-        rownum = 0
-        for rownum, record in enumerate(self.grid.records):
-            self.record_row(xlh, rownum, record)
-
-        # totals
-        if rownum and self.grid.subtotals != 'none' \
-                and self.grid.subtotal_cols:
-            self.totals_row(xlh, rownum + 1, self.grid.grand_totals)
-
-    def record_row(self, xlh, rownum, record):
-        for col in self.columns:
-            self.record_cell(xlh, col, record)
-        xlh.newrow()
-
-    def totals_row(self, xlh, rownum, record):
-        colspan = 0
-        firstcol = True
-        totals_xf = xlwt.easyxf('font: bold on; border: top thin')
-        for col in self.columns:
-            if col.key not in list(self.grid.subtotal_cols.keys()):
-                if firstcol:
-                    colspan += 1
-                else:
-                    xlh.awrite('', totals_xf)
-                continue
-            if firstcol:
-                numrecords = self.grid.record_count
-                bufferval = ngettext('Totals ({num} record):',
-                                     'Totals ({num} records):',
-                                     numrecords)
-                xlh.ws.write_merge(
-                    xlh.rownum,
-                    xlh.rownum,
-                    xlh.colnum,
-                    xlh.colnum + colspan - 1,
-                    fix_xls_value(bufferval),
-                    totals_xf
-                )
-                xlh.colnum = xlh.colnum + colspan
-                firstcol = False
-                colspan = 0
-            self.total_cell(xlh, col, record)
-        xlh.newrow()
-
-    def record_cell(self, xlh, col, record):
-        value = col.render('xls', record)
-        self.register_col_width(col, value)
-        stymat = col.xlwt_stymat_calc(value)
-        xlh.awrite(fix_xls_value(value), stymat)
-
-    def total_cell(self, xlh, col, record):
-        value = col.render('xls', record)
-        self.register_col_width(col, value)
-        stymat = col.xlwt_stymat_init()
-        stymat.font.bold = True
-        stymat.borders.top = xlwt.Formatting.Borders.THIN
-        xlh.awrite(fix_xls_value(value), stymat)
-
-    def file_name(self):
-        return '{0}_{1}.xls'.format(self.grid.ident, randnumerics(6))
-
-    def as_response(self, wb=None, sheet_name=None):
-        wb = self.build_sheet(wb, sheet_name)
-        buffer = io.BytesIO()
-        wb.save(buffer)
-        buffer.seek(0)
-        return self.grid.manager.file_as_response(buffer, self.file_name(), self.mime_type)
-
-
-class XLSX(GroupMixin, Renderer):
-    """Renderer for Excel XLSX output."""
-    mime_type = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
-
-    @property
-    def name(self):
-        return 'xlsx'
-
-    def init(self):
-        self.styles_cache = LazyDict()
+class XLSXWriterWorkbookManager:
+    def __init__(self, *args, **kwargs):
+        self.base_style_attrs = {
+            'bold': True,
+            'top': 6  # Double thick border
+        }
+        self._workbook = self.create_workbook()
         self._xlsx_format_cache = {}
-        self.default_style = {}
-        self.col_widths = {}
+        self.heading_style = self._workbook.add_format({'bold': True})
+        self.totals_style = self._workbook.add_format(self.base_style_attrs)
 
-    def get_xlsx_format(self, wb, style_dict):
+        self.styles_cache = LazyDict()
+        self.default_style = {}
+        super().__init__(*args, **kwargs)
+
+    def create_workbook(self):
+        buf = io.BytesIO()
+        return xlsxwriter.Workbook(buf, options={'in_memory': True})
+
+    def get_xlsx_format(self, style_dict):
         """
         This method is meant to solve a major performance issue with how xlsxwriter manages formats.
         Xlsxwriter maintains a cache of formats, however generating the cache key is surprisingly
@@ -1410,28 +1248,241 @@ class XLSX(GroupMixin, Renderer):
         """
         key = tuple(sorted(style_dict.items(), key=itemgetter(0)))
         if key not in self._xlsx_format_cache:
-            self._xlsx_format_cache[key] = wb.add_format(style_dict)
+            self._xlsx_format_cache[key] = self._workbook.add_format(style_dict)
         return self._xlsx_format_cache[key]
 
-    def style_for_column(self, wb, col):
-        """Return a column's style from cached information."""
+    def merged_totals_cell(self, xlh, value, style, colspan):
+        xlh.ws.merge_range(
+            xlh.rownum,
+            xlh.colnum,
+            xlh.rownum,
+            xlh.colnum + colspan - 1,
+            value,
+            style
+        )
+
+    def merged_heading_cell(self, xlh, value, style, col_index, colspan):
+        xlh.ws.merge_range(
+            0, col_index, 0, col_index + (colspan - 1),
+            value,
+            style
+        )
+
+    def style_for_column(self, col):
         if col.key not in self.styles_cache:
             style_dict = getattr(col, 'xlsx_style', self.default_style).copy()
             if col.xls_num_format:
                 style_dict['num_format'] = col.xls_num_format
-            self.styles_cache[col.key] = self.get_xlsx_format(wb, style_dict)
+            self.styles_cache[col.key] = self.get_xlsx_format(style_dict)
         return self.styles_cache[col.key]
 
+    def set_column_widths(self, writer, idx, width):
+        writer.ws.set_column(idx, idx, width)
+
+    def add_worksheet(self, sheet_name):
+        return self._workbook.add_worksheet(sheet_name)
+
+    @property
+    def fileclosed(self):
+        return self._workbook.fileclosed
+
+    def close(self):
+        self._workbook.close()
+
+    @property
+    def filename(self):
+        return self._workbook.filename
+
+    def write_group_header(self, xlh, col_index, data):
+        xlh.ws.write(0, col_index, data, self.heading_style)
+
+    def write_totals_row(self, xlh, col, value):
+        style = self.base_style_attrs.copy()
+        default_style = {}
+        style.update(getattr(col, 'xlsx_style', default_style))
+        style = self._workbook.add_format(style)
+        xlh.awrite(fix_xls_value(value), style)
+
+
+class OpenpyxlWorkbookManager:
+    def __init__(self, *args, **kwargs):
+        self._workbook = openpyxl.Workbook()
+        self._file = None
+        self._named_styles = set()
+        self.totals_style_spec = {
+            'font': Font(bold=True),
+            'border': Border(top=Side(border_style='double'))
+        }
+
+        self.heading_style = self.get_heading_style()
+        self.totals_style = self.get_totals_style()
+
+    def add_named_style(self, style: Dict[str, Union[Font, Border, Alignment]], name: str) -> str:
+        '''
+        Registers a named style based on the column key, and caches to avoid name conflicts later.
+        https://openpyxl.readthedocs.io/en/stable/styles.html#creating-a-named-style
+
+        Returns: string representing the name of the named style
+        '''
+
+        if name in self._named_styles:
+            return name
+
+        named_style = openpyxl.styles.NamedStyle(name=name)
+        for key, val in style.items():
+            setattr(named_style, key, val)
+        self._workbook.add_named_style(named_style)
+        self._named_styles.add(name)
+        return named_style.name
+
+    def get_heading_style(self):
+        return self.add_named_style({'font': Font(bold=True)}, 'heading')
+
+    def get_totals_style(self):
+        return self.add_named_style(self.totals_style_spec, 'totals')
+
+    @property
+    def filename(self):
+        '''
+        A misnomer but is consistent with xlsxwriter. This actually returns the file handle,
+        in this case a BytesIO
+        '''
+        if not self._file:
+            buf = io.BytesIO()
+            self._workbook.save(buf)
+            self._file = buf
+        return self._file
+
+    def close(self):
+        self._workbook.close()
+
+    @property
+    def fileclosed(self):
+        '''
+        Property does not exist in openpyxl like it does in xlsxwriter,
+        and trying to close an already closed book does not seem to cause issues.
+        '''
+        return False
+
+    def merged_totals_cell(self, xlh, value, style, colspan):
+        '''
+        openpyxl-specific creation of a merged cell for the totals row
+        '''
+        merged_cells = xlh.ws.cell(
+            row=xlh.rownum,
+            column=xlh.colnum
+        )
+        merged_cells.value = value
+        merged_cells.style = style
+        xlh.ws.merge_cells(
+            start_row=xlh.rownum,
+            start_column=xlh.colnum,
+            end_row=xlh.rownum,
+            end_column=xlh.colnum + colspan - 1
+        )
+
+    def merged_heading_cell(self, xlh, value, style, col_index, colspan):
+        '''
+        openpyxl-specific creation of a merged cell for group headers
+        '''
+        merged_cells = xlh.ws.cell(
+            row=1,
+            column=col_index + 1
+        )
+
+        # The editable cell within a merged cell (top left cell) is a Cell,
+        # while the other cells in that merge will yield a MergedCell, which
+        # has an immutable value
+        if isinstance(merged_cells, openpyxl.cell.Cell):
+            merged_cells.value = value
+
+        merged_cells.style = style
+        xlh.ws.merge_cells(
+            start_row=1,
+            start_column=col_index + 1,
+            end_row=1,
+            end_column=col_index + colspan
+        )
+
+    def style_for_column(self, col):
+        """Return a column's style from cached information."""
+        style_dict = getattr(col, 'xlsx_style', None)
+        if style_dict:
+            if 'num_format' in style_dict:
+                # translate the key for openpyxl
+                style_dict['number_format'] = style_dict['num_format']
+
+            # Only use certain keys, as they become style attributes
+            style_dict = {
+                key: style_dict[key] for key in ['font', 'alignment', 'border', 'number_format']
+                if key in style_dict
+            }
+            return self.add_named_style(style_dict, col.key)
+
+    def set_column_widths(self, writer, idx, width):
+        col_ltr = get_column_letter(idx + 1)
+        writer.ws.column_dimensions[col_ltr].width = width
+
+    def add_worksheet(self, sheet_name):
+        # Workbook is created with blank worksheet, just rename and return
+        sheet = self._workbook.worksheets[0]
+        sheet.title = sheet_name
+        return sheet
+
+    def write_group_header(self, xlh, col_index, data):
+        group_hdr_cell = xlh.ws.cell(row=1, column=col_index + 1)
+        group_hdr_cell.style = self.heading_style
+
+    def write_totals_row(self, xlh, col, value):
+        style = self.totals_style_spec.copy()
+        xlsx_style = getattr(col, 'xlsx_style', None)
+        if xlsx_style:
+            if 'num_format' in xlsx_style:
+                xlsx_style['number_format'] = xlsx_style['num_format']
+            style.update(xlsx_style)
+
+        named_style = self.add_named_style(style, f'{col.key}_totals')
+        xlh.awrite(fix_xls_value(value), named_style)
+
+
+class XLSX(GroupMixin, Renderer):
+    """Renderer for Excel XLSX output."""
+    mime_type = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
+
+    def __init__(self, grid):
+        self._manager_cls = None
+        super().__init__(grid)
+
+    def __call__(self, manager_cls=None):
+        self._manager_cls = manager_cls
+        return super().__call__()
+
+    @property
+    def name(self):
+        return 'xlsx'
+
+    def init(self):
+        self.col_widths = {}
+
+    def create_workbook(self):
+        if self._manager_cls:
+            return self._manager_cls()
+        if openpyxl:
+            return OpenpyxlWorkbookManager()
+        if xlsxwriter:
+            return XLSXWriterWorkbookManager()
+        raise ImportError('You must have either openpyxl or xlsxwriter installed.')
+
     def update_column_width(self, col, data):
         """Compute and store a column width from length of current data."""
         width = max((col.xls_width_calc(data), self.col_widths.get(col.key, 0)))
         self.col_widths[col.key] = width
 
-    def adjust_column_widths(self, writer):
+    def adjust_column_widths(self, writer, wb):
         """Apply stored column widths to the XLSX worksheet."""
         for idx, col in enumerate(self.columns):
             if col.key in self.col_widths:
-                writer.ws.set_column(idx, idx, self.col_widths[col.key])
+                wb.set_column_widths(writer, idx, self.col_widths[col.key])
 
     def build_sheet(self, wb=None, sheet_name=None):
         """Create and populate a worksheet for the current grid.
@@ -1447,15 +1498,12 @@ class XLSX(GroupMixin, Renderer):
         Returns:
             Workbook: Created/supplied workbook with the rendered worksheet added.
         """
-        if xlsxwriter is None:
-            raise ImportError('you must have xlsxwriter installed to use the XLSX renderer')
 
         if not self.can_render():
             raise RenderLimitExceeded('Unable to render XLSX sheet')
 
         if wb is None:
-            buf = io.BytesIO()
-            wb = xlsxwriter.Workbook(buf, options={'in_memory': True})
+            wb = self.create_workbook()
 
         sheet = wb.add_worksheet(self.sanitize_sheet_name(sheet_name or self.grid.ident))
         writer = WriterX(sheet)
@@ -1463,14 +1511,11 @@ class XLSX(GroupMixin, Renderer):
         self.sheet_header(writer, wb)
         self.sheet_body(writer, wb)
         self.sheet_footer(writer, wb)
-        self.adjust_column_widths(writer)
-
+        self.adjust_column_widths(writer, wb)
         return wb
 
     def render(self):
-        buf = io.BytesIO()
-        with xlsxwriter.Workbook(buf, options={'in_memory': True}) as wb:
-            return self.build_sheet(wb)
+        return self.build_sheet()
 
     def can_render(self):
         total_rows = self.grid.record_count + 1
@@ -1517,7 +1562,6 @@ class XLSX(GroupMixin, Renderer):
             xlh (WriterX): Helper for writing worksheet cells.
             wb (Workbook): xlsxwriter Workbook object for direct usage.
         """
-        heading_style = wb.add_format({'bold': True})
 
         # Render group labels above column headings.
         if self.has_groups():
@@ -1525,21 +1569,18 @@ class XLSX(GroupMixin, Renderer):
             for group, colspan in self.get_group_heading_colspans():
                 data = fix_xls_value(group.label) if group else None
                 if colspan == 1:
-                    xlh.awrite(data, heading_style)
-                    xlh.ws.write(0, col_index, data, heading_style)
+                    xlh.awrite(data, wb.heading_style)
+                    wb.write_group_header(xlh, col_index, data)
+
                 else:
-                    xlh.ws.merge_range(
-                        0, col_index, 0, col_index + (colspan - 1),
-                        data,
-                        heading_style
-                    )
+                    wb.merged_heading_cell(xlh, data, wb.heading_style, col_index, colspan)
 
                 col_index += colspan
 
             xlh.nextrow()
 
         for col in self.columns:
-            xlh.awrite(fix_xls_value(col.label), heading_style)
+            xlh.awrite(fix_xls_value(col.label), wb.heading_style)
             self.update_column_width(col, col.label)
         xlh.nextrow()
 
@@ -1576,7 +1617,7 @@ class XLSX(GroupMixin, Renderer):
         """
         for col in self.columns:
             value = col.render('xlsx', record)
-            style = self.style_for_column(wb, col)
+            style = wb.style_for_column(col)
             xlh.awrite(fix_xls_value(value), style)
             self.update_column_width(col, value)
         xlh.nextrow()
@@ -1585,17 +1626,13 @@ class XLSX(GroupMixin, Renderer):
         """Render a totals row based on subtotal columns defined in the grid."""
         colspan = 0
         firstcol = True
-        base_style_attrs = {
-            'bold': True,
-            'top': 6  # Double think border
-        }
-        base_style = wb.add_format(base_style_attrs)
+
         for col in self.columns:
             if col.key not in list(self.grid.subtotal_cols.keys()):
                 if firstcol:
                     colspan += 1
                 else:
-                    xlh.awrite('', base_style)
+                    xlh.awrite('', wb.totals_style)
                 continue
             if firstcol:
                 numrecords = self.grid.record_count
@@ -1604,26 +1641,16 @@ class XLSX(GroupMixin, Renderer):
                     's' if numrecords != 1 else '',
                 )
                 if colspan > 1:
-                    xlh.ws.merge_range(
-                        xlh.rownum,
-                        xlh.colnum,
-                        xlh.rownum,
-                        xlh.colnum + colspan - 1,
-                        bufferval,
-                        base_style
-                    )
+                    wb.merged_totals_cell(xlh, bufferval, wb.totals_style, colspan)
                     xlh.colnum = xlh.colnum + colspan
                 else:
-                    xlh.awrite(bufferval, base_style)
+                    xlh.awrite(bufferval, wb.totals_style)
 
                 firstcol = False
                 colspan = 0
 
-            style = base_style_attrs.copy()
-            style.update(getattr(col, 'xlsx_style', self.default_style))
-            style = wb.add_format(style)
             value = col.render('xlsx', record)
-            xlh.awrite(fix_xls_value(value), style)
+            wb.write_totals_row(xlh, col, value)
             self.update_column_width(col, value)
 
         xlh.nextrow()
