@@ -9,7 +9,6 @@ import io
 import arrow
 import openpyxl
 import pytest
-import xlrd
 import xlsxwriter
 from markupsafe import Markup
 from pyquery import PyQuery
@@ -32,10 +31,11 @@ from webgrid.renderers import (
     CSV,
     HTML,
     JSON,
-    XLS,
     XLSX,
     RenderLimitExceeded,
     render_html_attributes,
+    OpenpyxlWorkbookManager,
+    XLSXWriterWorkbookManager,
 )
 from webgrid_ta.grids import (
     ArrowCSVGrid,
@@ -291,17 +291,6 @@ class TestHtmlRenderer(object):
         tg = TGrid()
         assert 'Add Filter' not in tg.html()
 
-    @_inrequest('/')
-    def test_hide_excel_deprecated(self):
-        class TGrid(Grid):
-            hide_excel_link = True
-            Column('Test', Person.id)
-        with pytest.warns(
-            DeprecationWarning,
-            match='Hide excel link is deprecated, you should just override allowed_export_targets instead'  # noqa
-        ):
-            TGrid()
-
     def get_grid(self, **kwargs):
         g = SimpleGrid(**kwargs)
         g.set_records(self.key_data)
@@ -335,19 +324,10 @@ class TestHtmlRenderer(object):
         g = self.get_grid()
         assert '/thepage?dgreset=1&onpage=1&perpage=5' == g.html.current_url(dgreset=1)
 
-    @_inrequest('/thepage?perpage=5&onpage=1')
-    def test_xls_url(self):
-        g = self.get_grid()
-        with pytest.warns(DeprecationWarning,
-                          match='xls_url is deprecated. Use export_url instead.'):
-            url = g.html.xls_url()
-        assert url == '/thepage?export_to=xls&onpage=1&perpage=5'
-
     @_inrequest('/thepage?perpage=5&onpage=1')
     def test_export_url(self):
         g = self.get_grid()
         assert g.html.export_url('xlsx') == '/thepage?export_to=xlsx&onpage=1&perpage=5'
-        assert g.html.export_url('xls') == '/thepage?export_to=xls&onpage=1&perpage=5'
         assert g.html.export_url('csv') == '/thepage?export_to=csv&onpage=1&perpage=5'
 
     @_inrequest('/thepage?onpage=3')
@@ -737,15 +717,11 @@ class TestFooterRendersCorrectly(object):
         g = PeopleGrid()
         g.html
         assert '<a class="export-link" href="/?export_to=xlsx">XLSX</a>' in g.html()
-        assert '<a class="export-link" href="/?export_to=xls">XLS</a>' in g.html()
-        # make sure we are rendering the separator
-        assert '&nbsp;|' in g.html()
 
     @_inrequest('/')
     def test_people_html_footer_only_csv(self):
         g = PeopleCSVGrid()
         g.html
-        assert '<a class="export-link" href="/?export_to=xls">XLS</a>' not in g.html()
         assert '<a class="export-link" href="/?export_to=csv">CSV</a>' in g.html()
 
 
@@ -809,7 +785,6 @@ class TestJSONRenderer:
                     'full_name': 'Full Name',
                     'inactive': 'Active',
                     'numericcol': 'Number',
-                    'sortorder': 'Sort Order',
                     'state': 'State',
                     'status': 'Status',
                 },
@@ -829,9 +804,9 @@ class TestJSONRenderer:
                 },
                 'enable_search': True,
                 'enable_sort': True,
-                'export_targets': ['xls', 'xlsx'],
+                'export_targets': ['xlsx'],
                 'sortable_columns': [
-                    'firstname', 'inactive', 'status', 'createdts', 'due_date', 'sortorder',
+                    'firstname', 'inactive', 'status', 'createdts', 'due_date',
                     'state', 'numericcol', 'account_type',
                 ],
                 'filters': {
@@ -1052,91 +1027,21 @@ class TestJSONRenderer:
         ]
 
 
-class TestXLSRenderer(object):
-
-    def test_some_basics(self):
-        g = render_in_grid(PeopleGrid, 'xls')(per_page=1)
-        buffer = io.BytesIO()
-        wb = g.xls()
-        wb.save(buffer)
-        buffer.seek(0)
-
-        book = xlrd.open_workbook(file_contents=buffer.getvalue())
-        sh = book.sheet_by_name('render_in_grid')
-        # headers
-        assert sh.ncols == 10
-        assert sh.cell_value(0, 0) == 'First Name'
-        assert sh.cell_value(0, 7) == 'Sort Order'
-
-        # last data row
-        assert sh.cell_value(3, 0) == 'fn001'
-        assert sh.cell_value(3, 7) == 1
-        assert sh.nrows == 4
-
-    def test_subtotals_with_no_records(self):
-        g = PGGrandTotals()
-        g.column('firstname').filter.op = 'eq'
-        g.column('firstname').filter.value1 = 'foobar'
-        buffer = io.BytesIO()
-        wb = g.xls()
-        wb.save(buffer)
-        buffer.seek(0)
-
-    def test_long_grid_name(self):
-        class PeopleGridWithAReallyReallyLongName(PeopleGrid):
-            pass
-        g = PeopleGridWithAReallyReallyLongName()
-        buffer = io.BytesIO()
-        wb = g.xls()
-        wb.save(buffer)
-        buffer.seek(0)
-
-        book = xlrd.open_workbook(file_contents=buffer.getvalue())
-        book.sheet_by_name('people_grid_with_a_really_r...')
-
-    def test_can_render(self):
-        class FakeCountsGrid(PeopleGrid):
-            def __init__(self, record_count, col_count, has_subtotals):
-                self._num_records = record_count
-                self._col_count = col_count
-                self.subtotals = 'all' if has_subtotals else 'none'
-                super(FakeCountsGrid, self).__init__()
-
-            @property
-            def record_count(self):
-                return self._num_records
-
-            def iter_columns(self, render_type):
-                for _ in range(self._col_count):
-                    yield None
-
-        assert FakeCountsGrid(65535, 256, False).xls.can_render() is True
-        assert FakeCountsGrid(65536, 256, False).xls.can_render() is False
-        assert FakeCountsGrid(65535, 256, True).xls.can_render() is False
-        assert FakeCountsGrid(65534, 256, True).xls.can_render() is True
-        assert FakeCountsGrid(65535, 257, False).xls.can_render() is False
-
-    def test_render_error(self):
-        class Renderer(XLS):
-            def can_render(self):
-                return False
-
-        class TestGrid(PeopleGrid):
-            def set_renderers(self):
-                super(TestGrid, self).set_renderers()
-                self.xls = Renderer(self)
-
-        with pytest.raises(RenderLimitExceeded):
-            TestGrid().xls()
-
-
 class TestXLSXRenderer(object):
+    def test_using_xlsxwriter_library(self):
+        g = render_in_grid(PeopleGrid, 'xlsx')(per_page=1)
+        wb = g.xlsx(manager_cls=XLSXWriterWorkbookManager)
+        assert isinstance(wb._workbook, xlsxwriter.workbook.Workbook)
+
+    def test_using_openpyxl_library_default(self):
+        g = render_in_grid(PeopleGrid, 'xlsx')(per_page=1)
+        wb = g.xlsx()
+        assert isinstance(wb._workbook, openpyxl.Workbook)
 
     def test_some_basics(self):
         g = render_in_grid(PeopleGrid, 'xlsx')(per_page=1)
         wb = g.xlsx()
         wb.filename.seek(0)
-
         book = openpyxl.load_workbook(wb.filename)
         sh = book['render_in_grid']
 
@@ -1154,7 +1059,28 @@ class TestXLSXRenderer(object):
         grid = StopwatchGrid()
         wb = grid.xlsx()
         wb.filename.seek(0)
+        book = openpyxl.load_workbook(wb.filename)
+        sheet = book[book.sheetnames[0]]
+        #   [ A | B | C | D | E | F | G | H | I ]
+        # 1 [       | Lap 1 |   | Lap 2 | Lap 3 ]
+        row_values = [cell.value for cell in next(sheet.iter_rows(max_row=1))]
+        assert row_values == [None, None, 'Lap 1', None, None, 'Lap 2', None, 'Lap 3', None]
+        assert sheet.cell(2, 2).value == 'Label'
+        assert sheet.cell(3, 2).value == 'Watch 1'
+        assert [str(range_) for range_ in sheet.merged_cells.ranges] == [
+            'A1:B1',
+            'C1:D1',
+            # E is a single cell
+            'F1:G1',
+            'H1:I1',
+        ]
+        assert sheet.max_column == 9
 
+    def test_group_headings_xlsxwriter(self):
+        grid = StopwatchGrid()
+        wb = grid.xlsx(manager_cls=XLSXWriterWorkbookManager)
+        wb.filename.seek(0)
+        wb.close()
         book = openpyxl.load_workbook(wb.filename)
         sheet = book[book.sheetnames[0]]
         #   [ A | B | C | D | E | F | G | H | I ]
@@ -1179,17 +1105,43 @@ class TestXLSXRenderer(object):
         wb = g.xlsx()
         wb.filename.seek(0)
 
-    def test_long_grid_name(self):
+    def test_long_grid_name_xlsxwriter(self):
         class PeopleGridWithAReallyReallyLongName(PeopleGrid):
             pass
         g = PeopleGridWithAReallyReallyLongName()
-        wb = g.xlsx()
+        wb = g.xlsx(manager_cls=XLSXWriterWorkbookManager)
+        wb.close()
         wb.filename.seek(0)
 
         book = openpyxl.load_workbook(wb.filename)
         assert book['people_grid_with_a_really_r...']
 
-    def test_totals(self):
+    def test_long_grid_name_openpyxl(self):
+        class PeopleGridWithAReallyReallyLongName(PeopleGrid):
+            pass
+        g = PeopleGridWithAReallyReallyLongName()
+        wb = g.xlsx(manager_cls=OpenpyxlWorkbookManager)
+        wb.filename.seek(0)
+
+        book = openpyxl.load_workbook(wb.filename)
+        assert book['people_grid_with_a_really_r...']
+
+    def test_totals_xlsxwriter(self):
+        g = PeopleGrid()
+        g.subtotals = 'grand'
+
+        wb = g.xlsx(manager_cls=XLSXWriterWorkbookManager)
+        wb.filename.seek(0)
+        wb.close()
+
+        book = openpyxl.load_workbook(wb.filename)
+        sheet = book[book.sheetnames[0]]
+        assert sheet.max_row == 5
+        assert sheet.cell(5, 1).value == 'Totals (3 records):'
+        assert sheet.cell(5, 9).value == 6.39
+        assert [str(range_) for range_ in sheet.merged_cells.ranges] == ['A5:H5']
+
+    def test_totals_openpyxl(self):
         g = PeopleGrid()
         g.subtotals = 'grand'
 
@@ -1257,10 +1209,10 @@ class TestXLSXRenderer(object):
 
     def test_xlsx_format_caching(self):
         grid = PeopleGrid()
-        wb = xlsxwriter.Workbook()
-        format1 = grid.xlsx.style_for_column(wb, grid.column('status'))
-        format2 = grid.xlsx.style_for_column(wb, grid.column('state'))
-        format3 = grid.xlsx.style_for_column(wb, grid.column('numericcol'))
+        wb = XLSXWriterWorkbookManager()
+        format1 = wb.style_for_column(grid.column('status'))
+        format2 = wb.style_for_column(grid.column('state'))
+        format3 = wb.style_for_column(grid.column('numericcol'))
 
         assert format1 is not None
         assert format2 is not None
@@ -1272,17 +1224,19 @@ class TestXLSXRenderer(object):
         grid = PeopleGrid()
         grid.column('status').xlsx_style = {'bold': True, 'border': 1}
         grid.column('state').xlsx_style = {'bold': True, 'border': 2}
+        wb = XLSXWriterWorkbookManager()
 
-        format1 = grid.xlsx.style_for_column(wb, grid.column('status'))
-        format2 = grid.xlsx.style_for_column(wb, grid.column('state'))
+        format1 = wb.style_for_column(grid.column('status'))
+        format2 = wb.style_for_column(grid.column('state'))
         assert format1 is not format2
 
         grid = PeopleGrid()
         grid.column('status').xlsx_style = {'bold': True, 'border': 1}
         grid.column('state').xlsx_style = {'bold': True, 'border': 1}
+        wb = XLSXWriterWorkbookManager()
 
-        format1 = grid.xlsx.style_for_column(wb, grid.column('status'))
-        format2 = grid.xlsx.style_for_column(wb, grid.column('state'))
+        format1 = wb.style_for_column(grid.column('status'))
+        format2 = wb.style_for_column(grid.column('state'))
         assert format1 is format2
 
 
@@ -1385,21 +1339,3 @@ class TestArrowDate(object):
         })
         assert g.column('created_utc').filter.is_active
         g.record_count
-
-    def test_xls(self):
-        ArrowRecord.testing_create(created_utc=arrow.Arrow(2016, 8, 10, 1, 2, 3))
-        g = ArrowGrid()
-        buffer = io.BytesIO()
-        wb = g.xls()
-        wb.save(buffer)
-        buffer.seek(0)
-
-        book = xlrd.open_workbook(file_contents=buffer.getvalue())
-        sh = book.sheet_by_name('arrow_grid')
-        # headers
-        assert sh.cell_value(0, 0) == 'Created'
-        # data row
-        assert (
-            dt.datetime(*xlrd.xldate_as_tuple(sh.cell_value(1, 0), sh.book.datemode)[:6])
-            == dt.datetime(2016, 8, 10, 1, 2, 3)
-        )
